@@ -49,7 +49,9 @@ struct _CallsMMOrigin
   char *last_ussd_response;
 
   gulong           ussd_handle_id;
-  gchar *name;
+
+  char *id;
+  char *name;
   GHashTable *calls;
   char *country_code;
 };
@@ -68,11 +70,11 @@ G_DEFINE_TYPE_WITH_CODE (CallsMMOrigin, calls_mm_origin, G_TYPE_OBJECT,
 
 enum {
   PROP_0,
+  PROP_ID,
   PROP_NAME,
   PROP_CALLS,
   PROP_MODEM,
   PROP_COUNTRY_CODE,
-  PROP_NUMERIC,
   PROP_LAST_PROP,
 };
 static GParamSpec *props[PROP_LAST_PROP];
@@ -425,15 +427,15 @@ delete_call (CallsMMOrigin  *self,
 }
 
 static void
-call_state_changed_cb (CallsMMOrigin  *self,
-                       CallsCallState  new_state,
-                       CallsCallState  old_state,
-                       CallsCall      *call)
+call_state_changed_cb (CallsCall     *call,
+                       GParamSpec    *pspec,
+                       CallsMMOrigin *self)
 {
-  if (new_state != CALLS_CALL_STATE_DISCONNECTED)
-    {
-      return;
-    }
+  g_assert (CALLS_IS_MM_ORIGIN (self));
+  g_assert (CALLS_IS_MM_CALL (call));
+
+  if (calls_call_get_state (call) != CALLS_CALL_STATE_DISCONNECTED)
+    return;
 
   delete_call (self, CALLS_MM_CALL (call));
 }
@@ -448,9 +450,9 @@ add_call (CallsMMOrigin *self,
 
   call = calls_mm_call_new (mm_call);
 
-  g_signal_connect_swapped (call, "state-changed",
-                            G_CALLBACK (call_state_changed_cb),
-                            self);
+  g_signal_connect (call, "notify::state",
+                    G_CALLBACK (call_state_changed_cb),
+                    self);
 
   path = mm_call_dup_path (mm_call);
   g_hash_table_insert (self->calls, path, call);
@@ -641,6 +643,10 @@ set_property (GObject      *object,
   CallsMMOrigin *self = CALLS_MM_ORIGIN (object);
 
   switch (property_id) {
+  case PROP_ID:
+    self->id = g_value_dup_string (value);
+    break;
+
   case PROP_MODEM:
     g_set_object (&self->mm_obj, g_value_get_object(value));
     break;
@@ -661,6 +667,10 @@ get_property (GObject      *object,
   CallsMMOrigin *self = CALLS_MM_ORIGIN (object);
 
   switch (property_id) {
+  case PROP_ID:
+    g_value_set_string (value, self->id);
+    break;
+
   case PROP_NAME:
     g_value_set_string (value, self->name);
     break;
@@ -673,10 +683,6 @@ get_property (GObject      *object,
     g_value_set_string (value, self->country_code);
     break;
 
-  case PROP_NUMERIC:
-    g_value_set_boolean (value, TRUE);
-    break;
-
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
@@ -889,10 +895,10 @@ calls_mm_origin_class_init (CallsMMOriginClass *klass)
   g_object_class_override_property (object_class, ID, NAME);    \
   props[ID] = g_object_class_find_property(object_class, NAME);
 
+  IMPLEMENTS (PROP_ID, "id");
   IMPLEMENTS (PROP_NAME, "name");
   IMPLEMENTS (PROP_CALLS, "calls");
   IMPLEMENTS (PROP_COUNTRY_CODE, "country-code");
-  IMPLEMENTS (PROP_NUMERIC, "numeric-addresses");
 
 #undef IMPLEMENTS
 
@@ -934,10 +940,12 @@ calls_mm_origin_init (CallsMMOrigin *self)
 }
 
 CallsMMOrigin *
-calls_mm_origin_new (MMObject *mm_obj)
+calls_mm_origin_new (MMObject   *mm_obj,
+                     const char *id)
 {
   return g_object_new (CALLS_TYPE_MM_ORIGIN,
                        "mm-object", mm_obj,
+                       "id", id,
                        NULL);
 }
 
