@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021 Purism SPC
+ * Copyright (C) 2021-2022 Purism SPC
  *
  * This file is part of Calls.
  *
@@ -60,6 +60,7 @@
 enum {
   PROP_0,
   PROP_NAME,
+  PROP_ID,
   PROP_ACC_HOST,
   PROP_ACC_USER,
   PROP_ACC_PASSWORD,
@@ -74,7 +75,7 @@ enum {
   PROP_ACC_ADDRESS,
   PROP_CALLS,
   PROP_COUNTRY_CODE,
-  PROP_NUMERIC,
+  PROP_CAN_TEL,
   PROP_LAST_PROP,
 };
 static GParamSpec *props[PROP_LAST_PROP];
@@ -110,11 +111,15 @@ struct _CallsSipOrigin
   char *transport_protocol;
   gboolean auto_connect;
   gboolean direct_mode;
+  gboolean can_tel;
+
+  char *own_ip;
   gint local_port;
 
   const char *protocol_prefix;
   char *address;
   char *name;
+  char *id;
 
   GList *calls;
   GHashTable *call_handles;
@@ -132,6 +137,26 @@ G_DEFINE_TYPE_WITH_CODE (CallsSipOrigin, calls_sip_origin, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (CALLS_TYPE_ACCOUNT,
                                                 calls_sip_origin_accounts_interface_init))
 
+static void
+change_state (CallsSipOrigin          *self,
+              CallsAccountState        new_state,
+              CallsAccountStateReason  reason)
+{
+  CallsAccountState old_state;
+
+  g_assert (CALLS_SIP_ORIGIN (self));
+
+  if (self->state == new_state)
+    return;
+
+  old_state = self->state;
+  self->state = new_state;
+
+  g_signal_emit_by_name (self, "account-state-changed", old_state, new_state, reason);
+  calls_account_emit_message_for_state_change (CALLS_ACCOUNT (self), new_state, reason);
+}
+
+
 static void
 remove_call (CallsSipOrigin *self,
              CallsCall      *call,
@@ -188,20 +213,17 @@ remove_calls (CallsSipOrigin *self,
 
 
 static void
-on_call_state_changed (CallsSipOrigin *self,
-                       CallsCallState  new_state,
-                       CallsCallState  old_state,
-                       CallsCall      *call)
+on_call_state_changed (CallsSipCall   *call,
+                       GParamSpec     *pspec,
+                       CallsSipOrigin *self)
 {
   g_assert (CALLS_IS_SIP_ORIGIN (self));
   g_assert (CALLS_IS_CALL (call));
 
-  if (new_state != CALLS_CALL_STATE_DISCONNECTED)
-    {
-      return;
-    }
+  if (calls_call_get_state (CALLS_CALL (call)) != CALLS_CALL_STATE_DISCONNECTED)
+    return;
 
-  remove_call (self, call, "Disconnected");
+  remove_call (self, CALLS_CALL (call), "Disconnected");
 }
 
 
@@ -213,12 +235,25 @@ add_call (CallsSipOrigin *self,
 {
   CallsSipCall *sip_call;
   CallsCall *call;
+  CallsSipMediaPipeline *pipeline;
   g_autofree gchar *local_sdp = NULL;
+  g_auto (GStrv)  address_split = NULL;
+  const char *call_address = address;
+  gint rtp_port, rtcp_port;
 
-  /* TODO get free port by creating GSocket and passing that to the pipeline */
-  guint local_port = get_port_for_rtp ();
+  pipeline = calls_sip_media_manager_get_pipeline (self->media_manager);
+  rtp_port = calls_sip_media_pipeline_get_rtp_port (pipeline);
+  rtcp_port = calls_sip_media_pipeline_get_rtcp_port (pipeline);
 
-  sip_call = calls_sip_call_new (address, inbound, handle);
+  if (self->can_tel) {
+    address_split = g_strsplit_set (address, ":@;", -1);
+
+    if (g_strv_length (address_split) >=2 &&
+        g_strcmp0 (address_split[2], self->host) == 0)
+      call_address = address_split[1];
+  }
+
+  sip_call = calls_sip_call_new (call_address, inbound, self->own_ip, pipeline, handle);
   g_assert (sip_call != NULL);
 
   if (self->oper->call_handle)
@@ -232,23 +267,23 @@ add_call (CallsSipOrigin *self,
   call = CALLS_CALL (sip_call);
 
   g_signal_emit_by_name (CALLS_ORIGIN (self), "call-added", call);
-  g_signal_connect_swapped (call, "state-changed",
-                            G_CALLBACK (on_call_state_changed),
-                            self);
+  g_signal_connect (call, "notify::state",
+                    G_CALLBACK (on_call_state_changed),
+                    self);
 
   if (!inbound) {
-    calls_sip_call_setup_local_media_connection (sip_call, local_port, local_port + 1);
+    calls_sip_call_setup_local_media_connection (sip_call);
 
     local_sdp = calls_sip_media_manager_static_capabilities (self->media_manager,
-                                                             local_port,
-                                                             check_sips (address));
+                                                             self->own_ip,
+                                                             rtp_port,
+                                                             rtcp_port,
+                                                             FALSE);
 
     g_assert (local_sdp);
 
     g_debug ("Setting local SDP for outgoing call to %s:\n%s", address, local_sdp);
 
-    /* TODO transform tel URI according to https://tools.ietf.org/html/rfc3261#section-19.1.6 */
-
     /* TODO handle IPv4 vs IPv6 for nua_invite (SOATAG_TAG) */
     nua_invite (self->oper->call_handle,
                 SOATAG_AF (SOA_AF_IP4_IP6),
@@ -282,6 +317,14 @@ dial (CallsOrigin *origin,
     return;
   }
 
+  if (calls_account_get_state (CALLS_ACCOUNT (origin)) !=
+      CALLS_ACCOUNT_STATE_ONLINE) {
+
+    g_warning ("Tried dialing '%s' on origin '%s', but it's not online",
+               address, name);
+    return;
+  }
+
   self = CALLS_SIP_ORIGIN (origin);
 
   nh = nua_handle (self->nua, self->oper,
@@ -341,12 +384,10 @@ sip_authenticate (CallsSipOrigin *self,
   if (www_auth) {
     scheme = www_auth->au_scheme;
     realm = msg_params_find (www_auth->au_params, "realm=");
-  }
-  else if (proxy_auth) {
+  } else if (proxy_auth) {
     scheme = proxy_auth->au_scheme;
     realm = msg_params_find (proxy_auth->au_params, "realm=");
-  }
-  else {
+  } else {
     g_warning ("No authentication context found");
     return;
   }
@@ -376,19 +417,14 @@ sip_r_invite (int              status,
     /* TODO call states (see i_state) */
     if (status == 401 || status == 407) {
       sip_authenticate (origin, nh, sip);
-    }
-    else if (status == 403) {
+    } else if (status == 403) {
       g_warning ("Response to outgoing INVITE: 403 wrong credentials?");
-    }
-    else if (status == 904) {
+    } else if (status == 904) {
       g_warning ("Response to outgoing INVITE: 904 unmatched challenge."
                  "Possibly the challenge was already answered?");
-    }
-    else if (status == 180) {
-    }
-    else if (status == 100) {
-    }
-    else if (status == 200) {
+    } else if (status == 180) {
+    } else if (status == 100) {
+    } else if (status == 200) {
     }
 }
 
@@ -407,27 +443,31 @@ sip_r_register (int              status,
 
   if (status == 200) {
     g_debug ("REGISTER successful");
-    origin->state = CALLS_ACCOUNT_ONLINE;
+    change_state (origin,
+                  CALLS_ACCOUNT_STATE_ONLINE,
+                  CALLS_ACCOUNT_STATE_REASON_CONNECTED);
     nua_get_params (nua, TAG_ANY (), TAG_END());
+
     if (sip->sip_contact && sip->sip_contact->m_url && sip->sip_contact->m_url->url_host) {
-      calls_sip_media_manager_set_session_ip (origin->media_manager,
-                                              sip->sip_contact->m_url->url_host);
+      g_free (origin->own_ip);
+      origin->own_ip = g_strdup (sip->sip_contact->m_url->url_host);
     }
 
   } else if (status == 401 || status == 407) {
     sip_authenticate (origin, nh, sip);
-    origin->state = CALLS_ACCOUNT_AUTHENTICATING;
 
   } else if (status == 403) {
     g_warning ("wrong credentials?");
-    origin->state = CALLS_ACCOUNT_AUTHENTICATION_FAILURE;
+    change_state (origin,
+                  CALLS_ACCOUNT_STATE_OFFLINE,
+                  CALLS_ACCOUNT_STATE_REASON_AUTHENTICATION_FAILURE);
 
   } else if (status == 904) {
     g_warning ("unmatched challenge");
-    origin->state = CALLS_ACCOUNT_AUTHENTICATION_FAILURE;
+    change_state (origin,
+                  CALLS_ACCOUNT_STATE_ERROR,
+                  CALLS_ACCOUNT_STATE_REASON_INTERNAL_ERROR);
   }
-
-  g_object_notify_by_pspec (G_OBJECT (origin), props[PROP_ACC_STATE]);
 }
 
 
@@ -445,14 +485,15 @@ sip_r_unregister (int              status,
 
   if (status == 200) {
     g_debug ("Unregistering successful");
-    origin->state = CALLS_ACCOUNT_OFFLINE;
-
+    change_state (origin,
+                  CALLS_ACCOUNT_STATE_OFFLINE,
+                  CALLS_ACCOUNT_STATE_REASON_DISCONNECTED);
   } else {
     g_warning ("Unregisterung unsuccessful: %03d %s", status, phrase);
-    origin->state = CALLS_ACCOUNT_UNKNOWN_ERROR;
+    change_state (origin,
+                  CALLS_ACCOUNT_STATE_ERROR,
+                  CALLS_ACCOUNT_STATE_REASON_UNKNOWN);
   }
-
-  g_object_notify_by_pspec (G_OBJECT (origin), props[PROP_ACC_STATE]);
 }
 
 
@@ -468,6 +509,7 @@ sip_i_state (int              status,
 {
   const sdp_session_t *r_sdp = NULL;
   const sdp_session_t *l_sdp = NULL;
+  const char *r_sdp_str = NULL;
   gint call_state = nua_callstate_init;
   CallsCallState state;
   CallsSipCall *call;
@@ -490,6 +532,7 @@ sip_i_state (int              status,
   tl_gets (tags,
            SOATAG_LOCAL_SDP_REF (l_sdp),
            SOATAG_REMOTE_SDP_REF (r_sdp),
+           SOATAG_REMOTE_SDP_STR_REF (r_sdp_str),
            NUTAG_CALLSTATE_REF (call_state),
            NUTAG_OFFER_SENT_REF (offer_sent),
            NUTAG_OFFER_RECV_REF (offer_recv),
@@ -500,18 +543,20 @@ sip_i_state (int              status,
   if (status == 503) {
     CALLS_EMIT_MESSAGE (origin, "DNS error", GTK_MESSAGE_ERROR);
   }
-  /* XXX making some assumptions about the received SDP message here...
-   * namely: rtcp port = rtp port + 1
-   */
+
   if (r_sdp) {
     g_autoptr (GList) codecs =
       calls_sip_media_manager_get_codecs_from_sdp (origin->media_manager,
                                                    r_sdp->sdp_media);
     const char *session_ip = NULL;
     const char *media_ip = NULL;
+    int rtp_port;
+    int rtcp_port = 0;
+
+    g_debug ("Remote SDP was set to:\n%s", r_sdp_str);
 
     if (!codecs) {
-      g_warning ("No common codecs in SDP. Hanging up");
+      g_warning ("No common codecs in SDP. Hanging up. Remote SDP:\n%s", r_sdp_str);
       calls_call_hang_up (CALLS_CALL (call));
       return;
     }
@@ -530,10 +575,24 @@ sip_i_state (int              status,
     }
 
     calls_sip_call_set_codecs (call, codecs);
+
+    /* TODO This needs to adapt for the ICE case */
+    rtp_port = r_sdp->sdp_media->m_port;
+    for (sdp_attribute_t *attr = r_sdp->sdp_media->m_attributes; attr; attr = attr->a_next) {
+      if (g_strcmp0 (attr->a_name, "rtcp") == 0) {
+        rtcp_port = atoi (attr->a_value);
+        break;
+      }
+    }
+
+    /* Legacy fallback */
+    if (rtcp_port == 0)
+      rtcp_port = rtp_port + 1;
+
     calls_sip_call_setup_remote_media_connection (call,
                                                   media_ip ? : session_ip,
-                                                  r_sdp->sdp_media->m_port,
-                                                  r_sdp->sdp_media->m_port + 1);
+                                                  rtp_port,
+                                                  rtcp_port);
   }
 
   switch (call_state) {
@@ -569,7 +628,7 @@ sip_i_state (int              status,
     return;
   }
 
-  calls_sip_call_set_state (call, state);
+  calls_call_set_state (CALLS_CALL (call), state);
 }
 
 
@@ -638,10 +697,9 @@ sip_callback (nua_event_t   event,
     if (op->call_handle) {
       nua_respond (nh, 486, NULL, TAG_END ());
       g_debug ("Cannot handle more than one call. Rejecting");
-    }
-    else
+    } else {
       create_inbound (origin, from, nh);
-
+    }
     break;
 
   case nua_r_invite:
@@ -833,8 +891,6 @@ setup_nua (CallsSipOrigin *self)
     g_free (temp);
   }
 
-
-
   nua = nua_create (self->ctx->root,
                     sip_callback,
                     self,
@@ -856,6 +912,7 @@ setup_nua (CallsSipOrigin *self)
   return nua;
 }
 
+
 static char *
 get_registrar_url (CallsSipOrigin *self)
 {
@@ -867,6 +924,7 @@ get_registrar_url (CallsSipOrigin *self)
     return g_strconcat (self->protocol_prefix, ":", self->host, NULL);
 }
 
+
 static CallsSipHandles *
 setup_sip_handles (CallsSipOrigin *self)
 {
@@ -904,6 +962,9 @@ go_online (CallsAccount *account,
 
   self = CALLS_SIP_ORIGIN (account);
 
+  if (self->use_direct_connection)
+    return;
+
   if (!self->nua) {
     g_warning ("Cannot go online: nua handle not initialized");
     g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACC_STATE]);
@@ -913,7 +974,7 @@ go_online (CallsAccount *account,
   if (online) {
     g_autofree char *registrar_url = NULL;
 
-    if (self->state == CALLS_ACCOUNT_ONLINE)
+    if (self->state == CALLS_ACCOUNT_STATE_ONLINE)
       return;
 
     registrar_url = get_registrar_url (self);
@@ -925,7 +986,7 @@ go_online (CallsAccount *account,
                   TAG_END ());
 
   } else {
-    if (self->state == CALLS_ACCOUNT_OFFLINE)
+    if (self->state == CALLS_ACCOUNT_STATE_OFFLINE)
       return;
 
     nua_unregister (self->oper->register_handle,
@@ -933,6 +994,7 @@ go_online (CallsAccount *account,
   }
 }
 
+
 static const char *
 get_address (CallsAccount *account)
 {
@@ -989,6 +1051,12 @@ static gboolean
 init_sip_account (CallsSipOrigin *self,
                   GError        **error)
 {
+  g_assert (CALLS_IS_SIP_ORIGIN (self));
+
+  change_state (self,
+                CALLS_ACCOUNT_STATE_INITIALIZING,
+                CALLS_ACCOUNT_STATE_REASON_INITIALIZATION_STARTED);
+
   if (self->use_direct_connection) {
     g_debug ("Direct connection case. Using user and hostname");
     setup_account_for_direct_connection (self);
@@ -1000,18 +1068,22 @@ init_sip_account (CallsSipOrigin *self,
                  "init_sip_account (). "
                  "Try again when account is setup");
 
-    self->state = CALLS_ACCOUNT_NO_CREDENTIALS;
-    goto err;
+    change_state (self,
+                  CALLS_ACCOUNT_STATE_ERROR,
+                  CALLS_ACCOUNT_STATE_REASON_NO_CREDENTIALS);
+    return FALSE;
   }
 
-  // setup_nua() and setup_sip_handles() only after account data has been set
+  /* setup_nua() and setup_sip_handles() only after account data has been set */
   self->nua = setup_nua (self);
   if (self->nua == NULL) {
     g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                  "Failed setting up nua context");
 
-    self->state = CALLS_ACCOUNT_NULL;
-    goto err;
+    change_state (self,
+                  CALLS_ACCOUNT_STATE_ERROR,
+                  CALLS_ACCOUNT_STATE_REASON_INTERNAL_ERROR);
+    return FALSE;
   }
 
   self->oper = setup_sip_handles (self);
@@ -1019,26 +1091,21 @@ init_sip_account (CallsSipOrigin *self,
     g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                  "Failed setting operation handles");
 
-    self->state = CALLS_ACCOUNT_NULL;
-    goto err;
+    change_state (self,
+                  CALLS_ACCOUNT_STATE_ERROR,
+                  CALLS_ACCOUNT_STATE_REASON_INTERNAL_ERROR);
+    return FALSE;
   }
 
   /* In the case of a direct connection we're immediately good to go */
-  if (self->use_direct_connection)
-    self->state = CALLS_ACCOUNT_ONLINE;
-  else {
-    self->state = CALLS_ACCOUNT_OFFLINE;
+  change_state (self,
+                self->use_direct_connection ? CALLS_ACCOUNT_STATE_ONLINE : CALLS_ACCOUNT_STATE_OFFLINE,
+                CALLS_ACCOUNT_STATE_REASON_INITIALIZED);
 
-    if (self->auto_connect)
-      go_online (CALLS_ACCOUNT (self), TRUE);
-  }
+  if (self->auto_connect)
+    go_online (CALLS_ACCOUNT (self), TRUE);
 
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACC_STATE]);
   return TRUE;
-
- err:
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACC_STATE]);
-  return FALSE;
 }
 
 
@@ -1047,9 +1114,13 @@ deinit_sip_account (CallsSipOrigin *self)
 {
   g_assert (CALLS_IS_SIP_ORIGIN (self));
 
-  if (self->state == CALLS_ACCOUNT_NULL)
+  if (self->state == CALLS_ACCOUNT_STATE_UNKNOWN)
     return TRUE;
 
+  change_state (self,
+                CALLS_ACCOUNT_STATE_DEINITIALIZING,
+                CALLS_ACCOUNT_STATE_REASON_DEINITIALIZATION_STARTED);
+
   remove_calls (self, NULL);
 
   if (self->nua) {
@@ -1065,6 +1136,9 @@ deinit_sip_account (CallsSipOrigin *self)
 
     if (!self->is_shutdown_success) {
       g_warning ("nua_shutdown() timed out. Cannot proceed");
+      change_state (self,
+                    CALLS_ACCOUNT_STATE_ERROR,
+                    CALLS_ACCOUNT_STATE_REASON_INTERNAL_ERROR);
       return FALSE;
     }
     g_debug ("nua_shutdown() complete. Destroying nua handle");
@@ -1072,7 +1146,10 @@ deinit_sip_account (CallsSipOrigin *self)
     self->nua = NULL;
   }
 
-  self->state = CALLS_ACCOUNT_NULL;
+  g_clear_pointer (&self->own_ip, g_free);
+  change_state (self,
+                CALLS_ACCOUNT_STATE_UNKNOWN,
+                CALLS_ACCOUNT_STATE_REASON_DEINITIALIZED);
   return TRUE;
 }
 
@@ -1097,10 +1174,13 @@ supports_protocol (CallsOrigin *origin,
 
   if (g_strcmp0 (protocol, "sip") == 0)
     return TRUE;
+
   if (g_strcmp0 (protocol, "sips") == 0)
     return g_strcmp0 (self->protocol_prefix, "sips") == 0;
 
-  /* TODO need to set a property (from the UI) to allow using origin for telephony */
+  if (g_strcmp0 (protocol, "tel") == 0)
+    return self->can_tel;
+
   return FALSE;
 }
 
@@ -1128,6 +1208,10 @@ calls_sip_origin_set_property (GObject      *object,
   CallsSipOrigin *self = CALLS_SIP_ORIGIN (object);
 
   switch (property_id) {
+  case PROP_ID: /* contruct only */
+    self->id = g_value_dup_string (value);
+    break;
+
   case PROP_ACC_HOST:
     g_free (self->host);
     self->host = g_value_dup_string (value);
@@ -1179,6 +1263,10 @@ calls_sip_origin_set_property (GObject      *object,
     self->local_port = g_value_get_int (value);
     break;
 
+  case PROP_CAN_TEL:
+    self->can_tel = g_value_get_boolean (value);
+    break;
+
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
@@ -1199,6 +1287,11 @@ calls_sip_origin_get_property (GObject      *object,
   case PROP_NAME:
     g_value_set_string (value, self->name);
     break;
+
+  case PROP_ID:
+    g_value_set_string (value, self->id);
+    break;
+
   case PROP_ACC_HOST:
     g_value_set_string (value, self->host);
     break;
@@ -1251,8 +1344,8 @@ calls_sip_origin_get_property (GObject      *object,
     g_value_set_string (value, NULL);
     break;
 
-  case PROP_NUMERIC:
-    g_value_set_boolean (value, FALSE);
+  case PROP_CAN_TEL:
+    g_value_set_boolean (value, self->can_tel);
     break;
 
   default:
@@ -1295,7 +1388,15 @@ calls_sip_origin_dispose (GObject *object)
 {
   CallsSipOrigin *self = CALLS_SIP_ORIGIN (object);
 
-  if (!self->use_direct_connection && self->state == CALLS_ACCOUNT_ONLINE)
+  g_clear_pointer (&self->id, g_free);
+  g_clear_pointer (&self->own_ip, g_free);
+  g_clear_pointer (&self->transport_protocol, g_free);
+  g_clear_pointer (&self->display_name, g_free);
+  g_clear_pointer (&self->host, g_free);
+  g_clear_pointer (&self->user, g_free);
+  g_clear_pointer (&self->password, g_free);
+
+  if (!self->use_direct_connection && self->state == CALLS_ACCOUNT_STATE_ONLINE)
     go_online (CALLS_ACCOUNT (self), FALSE);
 
   deinit_sip_account (self);
@@ -1406,6 +1507,14 @@ calls_sip_origin_class_init (CallsSipOriginClass *klass)
                           G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
   g_object_class_install_property (object_class, PROP_SIP_CONTEXT, props[PROP_SIP_CONTEXT]);
 
+  props[PROP_CAN_TEL] =
+    g_param_spec_boolean ("can-tel",
+                          "Can telephone",
+                          "Whether to this account can be used for PSTN telephony",
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_CAN_TEL, props[PROP_CAN_TEL]);
+
   g_object_class_override_property (object_class, PROP_ACC_STATE, "account-state");
   props[PROP_ACC_STATE] = g_object_class_find_property (object_class, "account-state");
 
@@ -1416,10 +1525,10 @@ calls_sip_origin_class_init (CallsSipOriginClass *klass)
   g_object_class_override_property (object_class, ID, NAME);    \
   props[ID] = g_object_class_find_property(object_class, NAME);
 
+  IMPLEMENTS (PROP_ID, "id");
   IMPLEMENTS (PROP_NAME, "name");
   IMPLEMENTS (PROP_CALLS, "calls");
   IMPLEMENTS (PROP_COUNTRY_CODE, "country-code");
-  IMPLEMENTS (PROP_NUMERIC, "numeric-addresses");
 
 #undef IMPLEMENTS
 }
@@ -1472,6 +1581,7 @@ calls_sip_origin_set_credentials (CallsSipOrigin *self,
                                   const char     *display_name,
                                   const char     *transport_protocol,
                                   gint            port,
+                                  gboolean        can_tel,
                                   gboolean        auto_connect)
 {
   g_return_if_fail (CALLS_IS_SIP_ORIGIN (self));
@@ -1509,5 +1619,7 @@ calls_sip_origin_set_credentials (CallsSipOrigin *self,
 
   self->port = port;
 
+  self->can_tel = can_tel;
+
   recreate_sip (self);
 }
