@@ -83,6 +83,16 @@ struct _CallsSipProvider
 static void calls_sip_provider_message_source_interface_init (CallsMessageSourceInterface *iface);
 static void calls_sip_provider_account_provider_interface_init (CallsAccountProviderInterface *iface);
 
+#ifdef FOR_TESTING
+
+G_DEFINE_TYPE_WITH_CODE
+(CallsSipProvider, calls_sip_provider, CALLS_TYPE_PROVIDER,
+ G_IMPLEMENT_INTERFACE (CALLS_TYPE_MESSAGE_SOURCE,
+                        calls_sip_provider_message_source_interface_init)
+ G_IMPLEMENT_INTERFACE (CALLS_TYPE_ACCOUNT_PROVIDER,
+                        calls_sip_provider_account_provider_interface_init))
+
+#else
 
 G_DEFINE_DYNAMIC_TYPE_EXTENDED
 (CallsSipProvider, calls_sip_provider, CALLS_TYPE_PROVIDER, 0,
@@ -91,6 +101,8 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED
  G_IMPLEMENT_INTERFACE_DYNAMIC (CALLS_TYPE_ACCOUNT_PROVIDER,
                                 calls_sip_provider_account_provider_interface_init))
 
+#endif /* FOR_TESTING */
+
 typedef struct {
   CallsSipProvider *provider;
   GKeyFile         *key_file;
@@ -104,6 +116,7 @@ on_origin_pw_looked_up (GObject      *source,
 {
   SipOriginLoadData *data;
   g_autoptr (GError) error = NULL;
+  g_autofree char *id = NULL;
   g_autofree char *name = NULL;
   g_autofree char *host = NULL;
   g_autofree char *user = NULL;
@@ -114,11 +127,17 @@ on_origin_pw_looked_up (GObject      *source,
   gint local_port = 0;
   gboolean auto_connect = TRUE;
   gboolean direct_mode = FALSE;
+  gboolean can_tel = FALSE;
 
   g_assert (user_data);
 
   data = user_data;
 
+  if (g_key_file_has_key (data->key_file, data->name, "Id", NULL))
+    id = g_key_file_get_string (data->key_file, data->name, "Id", NULL);
+  else
+    id = g_strdup (data->name);
+
   host = g_key_file_get_string (data->key_file, data->name, "Host", NULL);
   user = g_key_file_get_string (data->key_file, data->name, "User", NULL);
   display_name = g_key_file_get_string (data->key_file, data->name, "DisplayName", NULL);
@@ -136,6 +155,10 @@ on_origin_pw_looked_up (GObject      *source,
   if (g_key_file_has_key (data->key_file, data->name, "DirectMode", NULL))
     direct_mode = g_key_file_get_boolean (data->key_file, data->name, "DirectMode", NULL);
 
+  if (g_key_file_has_key (data->key_file, data->name, "CanTel", NULL))
+    can_tel =
+      g_key_file_get_boolean (data->key_file, data->name, "CanTel", NULL);
+
   /* PW */
   password = secret_password_lookup_finish (result, &error);
   if (!direct_mode && error) {
@@ -155,6 +178,7 @@ on_origin_pw_looked_up (GObject      *source,
 #undef IS_NULL_OR_EMPTY
 
   calls_sip_provider_add_origin_full (data->provider,
+                                      id,
                                       host,
                                       user,
                                       password,
@@ -164,6 +188,7 @@ on_origin_pw_looked_up (GObject      *source,
                                       auto_connect,
                                       direct_mode,
                                       local_port,
+                                      can_tel,
                                       FALSE);
 }
 static void
@@ -255,6 +280,7 @@ origin_to_keyfile (CallsSipOrigin *origin,
                    GKeyFile       *key_file,
                    const char     *name)
 {
+  g_autofree char *id = NULL;
   g_autofree char *host = NULL;
   g_autofree char *user = NULL;
   g_autofree char *password = NULL;
@@ -265,11 +291,13 @@ origin_to_keyfile (CallsSipOrigin *origin,
   gint local_port;
   gboolean auto_connect;
   gboolean direct_mode;
+  gboolean can_tel;
 
   g_assert (CALLS_IS_SIP_ORIGIN (origin));
   g_assert (key_file);
 
   g_object_get (origin,
+                "id", &id,
                 "host", &host,
                 "user", &user,
                 "password", &password,
@@ -279,8 +307,10 @@ origin_to_keyfile (CallsSipOrigin *origin,
                 "auto-connect", &auto_connect,
                 "direct-mode", &direct_mode,
                 "local-port", &local_port,
+                "can-tel", &can_tel,
                 NULL);
 
+  g_key_file_set_string (key_file, name, "Id", id);
   g_key_file_set_string (key_file, name, "Host", host);
   g_key_file_set_string (key_file, name, "User", user);
   g_key_file_set_string (key_file, name, "DisplayName", display_name ?: "");
@@ -289,9 +319,9 @@ origin_to_keyfile (CallsSipOrigin *origin,
   g_key_file_set_boolean (key_file, name, "AutoConnect", auto_connect);
   g_key_file_set_boolean (key_file, name, "DirectMode", direct_mode);
   g_key_file_set_integer (key_file, name, "LocalPort", local_port);
+  g_key_file_set_boolean (key_file, name, "CanTel", can_tel);
 
-  label_secret = g_strdup_printf ("Calls Password for %s",
-                                  calls_account_get_address (CALLS_ACCOUNT (origin)));
+  label_secret = g_strdup_printf ("Calls Password for %s", id);
 
   /* save to keyring */
   secret_password_store (calls_secret_get_schema (), NULL, label_secret, password,
@@ -493,11 +523,14 @@ calls_sip_provider_dispose (GObject *object)
 }
 
 
+#ifndef FOR_TESTING
+
 static void
 calls_sip_provider_class_finalize (CallsSipProviderClass *klass)
 {
 }
 
+#endif /* FOR_TESTING */
 
 static void
 calls_sip_provider_class_init (CallsSipProviderClass *klass)
@@ -608,6 +641,7 @@ calls_sip_provider_init (CallsSipProvider *self)
 /**
  * calls_sip_provider_add_origin:
  * @self: A #CallsSipProvider
+ * @id: The id of the new origin (should be unique)
  * @host: The host to connect to
  * @user: The username to use
  * @password: The password to use
@@ -621,6 +655,7 @@ calls_sip_provider_init (CallsSipProvider *self)
  */
 CallsSipOrigin *
 calls_sip_provider_add_origin (CallsSipProvider *self,
+                               const char       *id,
                                const char       *host,
                                const char       *user,
                                const char       *password,
@@ -630,6 +665,7 @@ calls_sip_provider_add_origin (CallsSipProvider *self,
                                gboolean          store_credentials)
 {
   return calls_sip_provider_add_origin_full (self,
+                                             id,
                                              host,
                                              user,
                                              password,
@@ -639,12 +675,14 @@ calls_sip_provider_add_origin (CallsSipProvider *self,
                                              TRUE,
                                              FALSE,
                                              0,
+                                             FALSE,
                                              store_credentials);
 }
 
 /**
  * calls_sip_provider_add_origin_full:
  * @self: A #CallsSipProvider
+ * @id: The id of the new origin (should be unique)
  * @host: The host to connect to
  * @user: The username to use
  * @password: The password to use
@@ -653,6 +691,7 @@ calls_sip_provider_add_origin (CallsSipProvider *self,
  * @auto_connect: Whether to automatically try going online
  * @direct_mode: Whether to use direct connection mode. Useful when you don't want to
  * connect to a SIP server. Mostly useful for testing and debugging.
+ * @can_tel: Whether this origin can be used for PSTN telephony
  * @store_credentials: Whether to store credentials for this origin to disk
  *
  * Adds a new origin (SIP account). If @direct_mode is %TRUE then @host, @user and
@@ -662,6 +701,7 @@ calls_sip_provider_add_origin (CallsSipProvider *self,
  */
 CallsSipOrigin *
 calls_sip_provider_add_origin_full (CallsSipProvider *self,
+                                    const char       *id,
                                     const char       *host,
                                     const char       *user,
                                     const char       *password,
@@ -671,12 +711,14 @@ calls_sip_provider_add_origin_full (CallsSipProvider *self,
                                     gboolean          auto_connect,
                                     gboolean          direct_mode,
                                     gint              local_port,
+                                    gboolean          can_tel,
                                     gboolean          store_credentials)
 {
   g_autoptr (CallsSipOrigin) origin = NULL;
   g_autofree char *protocol = NULL;
 
   g_return_val_if_fail (CALLS_IS_SIP_PROVIDER (self), NULL);
+  g_return_val_if_fail (id || *id, NULL);
 
   /* direct-mode is mostly useful for testing without a SIP server */
   if (!direct_mode) {
@@ -692,6 +734,7 @@ calls_sip_provider_add_origin_full (CallsSipProvider *self,
   }
 
   origin = g_object_new (CALLS_TYPE_SIP_ORIGIN,
+                         "id", id,
                          "sip-context", self->ctx,
                          "host", host,
                          "user", user,
@@ -702,6 +745,7 @@ calls_sip_provider_add_origin_full (CallsSipProvider *self,
                          "auto-connect", auto_connect,
                          "direct-mode", direct_mode,
                          "local-port", local_port,
+                         "can-tel", can_tel,
                          NULL);
 
   g_list_store_append (self->origins, origin);
@@ -800,6 +844,7 @@ calls_sip_provider_save_accounts_to_disk (CallsSipProvider *self)
 }
 
 
+#ifndef FOR_TESTING
 
 G_MODULE_EXPORT void
 peas_register_types (PeasObjectModule *module)
@@ -810,3 +855,5 @@ peas_register_types (PeasObjectModule *module)
                                               CALLS_TYPE_PROVIDER,
                                               CALLS_TYPE_SIP_PROVIDER);
 }
+
+#endif /* FOR_TESTING */
