@@ -28,20 +28,22 @@
 #define G_LOG_DOMAIN "CallsApplication"
 
 #include "config.h"
+
+#include "calls-application.h"
+#include "calls-call-window.h"
 #include "calls-dbus-manager.h"
 #include "calls-history-box.h"
-#include "calls-new-call-box.h"
-#include "calls-encryption-indicator.h"
-#include "calls-ringer.h"
-#include "calls-notifier.h"
-#include "calls-record-store.h"
-#include "calls-call-window.h"
+#include "calls-log.h"
 #include "calls-main-window.h"
 #include "calls-manager.h"
-#include "calls-application.h"
-#include "calls-log.h"
+#include "calls-message-source.h"
+#include "calls-new-call-box.h"
+#include "calls-notifier.h"
+#include "calls-record-store.h"
+#include "calls-ringer.h"
 #include "version.h"
 
+#include <call-ui.h>
 #include <glib/gi18n.h>
 #include <handy.h>
 #include <libcallaudio.h>
@@ -281,6 +283,51 @@ call_number (CallsApplication *self,
 }
 
 
+static void
+open_sip_uri (CallsApplication *self,
+              const char       *uri)
+{
+  g_auto (GStrv) tokens = NULL;
+  g_assert (uri);
+
+  tokens = g_strsplit (uri, "///", 2);
+
+  if (g_strv_length (tokens) == 2) {
+    /* Remove "///" from "sip:///user@host" */
+    g_autofree char *dial_string = g_strconcat (tokens[0], tokens[1], NULL);
+
+    calls_main_window_dial (self->main_window, dial_string);
+  } else {
+    /* Dial the uri as it is */
+    calls_main_window_dial (self->main_window, uri);
+  }
+}
+
+
+static void
+open_tel_uri (CallsApplication *self,
+              const char       *uri)
+{
+  const char *number = NULL;
+
+  g_debug ("Opening tel URI `%s'", uri);
+
+  number = &uri[4]; // tel:NUMBER
+  if (!number || !*number) {
+    g_autofree char *msg =
+      g_strdup_printf (_("Tried dialing invalid tel URI `%s'"), uri);
+
+    calls_message_source_emit_message (CALLS_MESSAGE_SOURCE (calls_manager_get_default ()),
+                                       "msg",
+                                       GTK_MESSAGE_WARNING);
+    g_warning ("Ignoring invalid tel URI `%s'", uri);
+    return;
+  }
+
+  call_number (self, number);
+}
+
+
 static void
 dial_action (GSimpleAction *action,
              GVariant      *parameter,
@@ -292,7 +339,13 @@ dial_action (GSimpleAction *action,
   number = g_variant_get_string (parameter, NULL);
   g_return_if_fail (number != NULL);
 
-  call_number (self, number);
+  if (g_str_has_prefix (number, "sip:") ||
+      g_str_has_prefix (number, "sips:"))
+    open_sip_uri (self, number);
+  else if (g_str_has_prefix (number, "tel:"))
+    open_tel_uri (self, number);
+  else
+    call_number (self, number);
 }
 
 static void
@@ -322,9 +375,11 @@ static void
 manager_state_changed_cb (GApplication *application)
 {
   GAction* dial_action = g_action_map_lookup_action (G_ACTION_MAP (application), "dial");
-  CallsManagerState state = calls_manager_get_state (calls_manager_get_default ());
+  CallsManagerFlags state_flags = calls_manager_get_state_flags (calls_manager_get_default ());
+  gboolean enabled = (state_flags & CALLS_MANAGER_FLAGS_HAS_CELLULAR_MODEM) ||
+    (state_flags & CALLS_MANAGER_FLAGS_HAS_VOIP_ACCOUNT);
 
-  g_simple_action_set_enabled (G_SIMPLE_ACTION (dial_action), state == CALLS_MANAGER_STATE_READY);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (dial_action), enabled);
 }
 
 static const GActionEntry actions[] =
@@ -358,15 +413,24 @@ startup (GApplication *application)
 {
   g_autoptr (GtkCssProvider) provider = NULL;
   g_autoptr (GError) error = NULL;
+#if HDY_CHECK_VERSION(1, 5, 0)
+  HdyStyleManager *style_manager;
+#endif
 
   G_APPLICATION_CLASS (calls_application_parent_class)->startup (application);
 
   hdy_init ();
 
+#if HDY_CHECK_VERSION(1, 5, 0)
+  style_manager = hdy_style_manager_get_default ();
+
+  hdy_style_manager_set_color_scheme (style_manager, HDY_COLOR_SCHEME_PREFER_LIGHT);
+#endif
+
   if (!call_audio_init (&error))
-    {
-      g_warning ("Failed to init libcallaudio: %s", error->message);
-    }
+    g_warning ("Failed to init libcallaudio: %s", error->message);
+
+  cui_init (TRUE);
 
   g_set_prgname (APP_ID);
   g_set_application_name (_("Calls"));
@@ -440,6 +504,15 @@ calls_application_command_line (GApplication            *application,
   return 0;
 }
 
+static void
+app_shutdown (GApplication *application)
+{
+  cui_uninit ();
+
+  G_APPLICATION_CLASS (calls_application_parent_class)->shutdown (application);
+}
+
+
 static void
 notify_window_visible_cb (GtkWidget       *window,
                           GParamSpec      *pspec,
@@ -492,51 +565,6 @@ start_proper (CallsApplication  *self)
   return TRUE;
 }
 
-static void
-open_sip_uri (CallsApplication *self,
-              const char       *uri)
-{
-  char **tokens = NULL;
-  g_assert (uri);
-
-  tokens = g_strsplit (uri, "///", 2);
-
-  if (tokens) {
-    /* Remove "///" from "sip:///user@host" */
-    g_autofree char *dial_string = g_strconcat (tokens[0], tokens[1], NULL);
-
-    calls_main_window_dial (self->main_window, dial_string);
-
-    g_strfreev (tokens);
-  } else {
-    /* Dial the uri as it is */
-    calls_main_window_dial (self->main_window, uri);
-  }
-}
-
-static void
-open_tel_uri (CallsApplication *self,
-              const char       *uri)
-{
-  const char *number = NULL;
-
-  g_debug ("Opening tel URI `%s'", uri);
-
-  number = &uri[4]; // tel:NUMBER
-  if (!number || number[0] == '\0') {
-    g_autofree char *msg =
-      g_strdup_printf (_("Tried invalid tel URI `%s'"), uri);
-
-    g_signal_emit_by_name (calls_manager_get_default (),
-                           "error",
-                           msg);
-    g_warning ("Ignoring invalid tel URI `%s'", uri);
-    return;
-  }
-
-  call_number (self, number);
-}
-
 static void
 activate (GApplication *application)
 {
@@ -604,8 +632,9 @@ app_open (GApplication  *application,
 
     msg = g_strdup_printf (_("Don't know how to open `%s'"), uri);
 
-    g_signal_emit_by_name (calls_manager_get_default (),
-                           "error", msg);
+    calls_message_source_emit_message (CALLS_MESSAGE_SOURCE (calls_manager_get_default ()),
+                                       msg,
+                                       GTK_MESSAGE_WARNING);
   }
 }
 
@@ -637,12 +666,12 @@ calls_application_class_init (CallsApplicationClass *klass)
   application_class->handle_local_options = calls_application_handle_local_options;
   application_class->startup = startup;
   application_class->command_line = calls_application_command_line;
+  application_class->shutdown = app_shutdown;
   application_class->activate = activate;
   application_class->open = app_open;
   application_class->dbus_register  = calls_application_dbus_register;
   application_class->dbus_unregister  = calls_application_dbus_unregister;
 
-  g_type_ensure (CALLS_TYPE_ENCRYPTION_INDICATOR);
   g_type_ensure (CALLS_TYPE_HISTORY_BOX);
   g_type_ensure (CALLS_TYPE_NEW_CALL_BOX);
 }
